package com.gx.obe.server.im.util;

import java.math.BigDecimal;

import org.slf4j.Logger;


/**
 * 由于Java的简单类型不能够精确的对浮点数进行运算，这个工具类提供精
 * 确的浮点数运算，包括加减乘除和四舍五入。
 */

public class BigDecimalUtil {

	private static Logger LOG = org.slf4j.LoggerFactory.getLogger(BigDecimalUtil.class);
	// 默认除法运算精度

	private static final int DEF_DIV_SCALE = 10;

	// 这个类不能实例化

	private BigDecimalUtil() {

	}

	/**
	 * @Description:  提供精确的加法运算。
	 * @author guoyr
	 * @param v1 被加数
	 * @param v2 加数
	 * @return 两个参数的和
	 */
	public static BigDecimal add(double v1, double v2) {

		BigDecimal b1;
		try {
			b1 = new BigDecimal(Double.toString(v1));
		} catch (Exception e) {
			b1 = null;
			LOG.error(e.getMessage(), e);
		}

		BigDecimal b2;
		try {
			b2 = new BigDecimal(Double.toString(v2));
		} catch (Exception e) {
			b2 = null;
			LOG.error(e.getMessage(), e);
		}

		return add(b1, b2);
	}

	/**
	 * @Description:  提供精确的加法运算。
	 * @author guoyr
	 * @param b1 被加数
	 * @param b2 加数
	 * @return 两个参数的和
	 */
	public static BigDecimal add(BigDecimal b1, BigDecimal b2) {
		if (null == b1) {
			return b2;
//			b1 = BigDecimal.ZERO;
		}
		if (null == b2) {
			return b1;
//			b2 = BigDecimal.ZERO;
		}
		return b1.add(b2);
	}
	
	/**
	 * @Description:  提供精确的减法运算。
	 * @author guoyr
	 * @param v1 被减数
	 * @param v2 减数
	 * @return 两个参数的差
	 */
	public static BigDecimal sub(double v1, double v2) {

		BigDecimal b1;
		try {
			b1 = new BigDecimal(Double.toString(v1));
		} catch (Exception e) {
			b1 = null;
			LOG.error(e.getMessage(), e);
		}
		BigDecimal b2;
		try {
			b2 = new BigDecimal(Double.toString(v2));
		} catch (Exception e) {
			b2 = null;
			LOG.error(e.getMessage(), e);
		}

		return sub(b1, b2);

	}

	
	/**
	 * @Description:  提供精确的减法运算。
	 * @author guoyr
	 * @param b1  被减数
	 * @param b2 减数
	 * @return 两个参数的差
	 */
	public static BigDecimal sub(BigDecimal b1, BigDecimal b2) {
		if (null == b1) {
			return b2;
//			b1 = BigDecimal.ZERO;
		}
		if (null == b2) {
			return b1;
//			b2 = BigDecimal.ZERO;
		}
		return b1.subtract(b2);
	}

	/**
	 * @Description:  提供精确的乘法运算。
	 * @author guoyr
	 * @param v1 被乘数
	 * @param v2 乘数
	 * @return 两个参数的积
	 */
	public static BigDecimal mul(double v1, double v2) {
		BigDecimal b1;
		try {
			b1 = new BigDecimal(Double.toString(v1));
		} catch (Exception e) {
			b1 = null;
//			b1 = BigDecimal.ZERO;
			LOG.error(e.getMessage(), e);
		}
		BigDecimal b2;
		try {
			b2 = new BigDecimal(Double.toString(v2));
		} catch (Exception e) {
			b2 = null;//BigDecimal.ZERO;
			LOG.error(e.getMessage(), e);
		}
		return mul(b1, b2);

	}

	/**
	 * @Description:  提供精确的乘法运算。
	 * @author guoyr
	 * @param b1 被乘数
	 * @param b2 乘数
	 * @return 两个参数的积
	 */
	public static BigDecimal mul(BigDecimal b1, BigDecimal b2) {
		if (null == b1) {
			return b2;
		}
		if (null == b2) {
			return b1;
		}
		return b1.multiply(b2);
	}

	/**
	 * @Description:  提供（相对）精确的除法运算，当发生除不尽的情况时，精确到小数点以后10位，以后的数字四舍五入。
	 * @author guoyr
	 * @param v1 被除数
	 * @param v2 除数
	 * @return 两个参数的商
	 */
	public static BigDecimal div(double v1, double v2) {
		return div(v1, v2, DEF_DIV_SCALE);
	}

	
	/**
	 * @Description:  提供（相对）精确的除法运算，当发生除不尽的情况时，精确到小数点以后10位，以后的数字四舍五入。
	 * @author guoyr
	 * @param v1 v1 被除数
	 * @param v2 v2 除数
	 * @return 两个参数的商
	 */
	public static BigDecimal div(BigDecimal v1, BigDecimal v2) {
		return div(v1, v2, DEF_DIV_SCALE);
	}
	
	/**
	 * @Description:  提供（相对）精确的除法运算。当发生除不尽的情况时，由scale参数指
	 * 定精度，以后的数字四舍五入。
	 * @author guoyr
	 * @param v1 被除数
	 * @param v2 除数
	 * @param scale 表示表示需要精确到小数点以后几位。
	 * @return 两个参数的商
	 */
	public static BigDecimal div(double v1, double v2, int scale) {

		if (scale < 0) {
			// throw new IllegalArgumentException(
			// "The scale must be a positive integer or zero");
			scale = DEF_DIV_SCALE;
		}

		BigDecimal b1;
		try {
			b1 = new BigDecimal(Double.toString(v1));
		} catch (Exception e) {
			b1 = null;
			LOG.error(e.getMessage(), e);
		}
		BigDecimal b2;
		try {
			b2 = new BigDecimal(Double.toString(v2));
		} catch (Exception e) {
			b2 = null;
			LOG.error(e.getMessage(), e);
		}
		return div(b1, b2, scale);
	}

	
	/**
	 * @Description:  提供（相对）精确的除法运算
	 * @author guoyr
	 * @param b1 被除数
	 * @param b2  除数
	 * @param scale 表示表示需要精确到小数点以后几位。
	 * @return 两个参数的商
	 */
	public static BigDecimal div(BigDecimal b1, BigDecimal b2, int scale) {
		if (null == b1) {
//			b1 = BigDecimal.ZERO;
			return null;
		}
		if (null == b2) {
			return null;
		}else if(b2.compareTo(BigDecimal.ZERO) == 0){
			return null;
		}
		return b1.divide(b2, scale, BigDecimal.ROUND_HALF_UP);

	}

	/**
	 * @Description:  提供精确的小数位四舍五入处理。
	 * @author guoyr
	 * @param v 需要四舍五入的数字
	 * @param scale 小数点后保留几位
	 * @return 四舍五入后的结果
	 */
	public static BigDecimal round(double v, int scale) {
		BigDecimal b;
		try {
			b = new BigDecimal(Double.toString(v));
		} catch (Exception e) {
			b = null;
			LOG.error(e.getMessage(), e);
		}
		return round(b, scale);
	}

	
	/**
	 * @Description:  供精确的小数位四舍五入处理。
	 * @author guoyr
	 * @param b 需要四舍五入的数字
	 * @param scale 小数点后保留几位
	 * @return 四舍五入后的结果
	 */
	public static BigDecimal round(BigDecimal b, int scale) {

		if (scale < 0) {

			scale = 0;
		}
		if (null == b) {
			return null;
		}
		return b.setScale(scale, BigDecimal.ROUND_HALF_UP);

		// String fmtString = "0000000000000000" ; //16bit
		// fmtString = scale>0 ? "0." + fmtString.substring(0,scale):"0" ;
		// DecimalFormat dFormat = new DecimalFormat(fmtString);
		// return dFormat.format(v);

		// BigDecimal one = new BigDecimal("1");
		//
		// return b.divide(one, scale, BigDecimal.ROUND_HALF_UP);
	}

	public static void main(String args[]) {
	}
};
